//
// Fl_Table_Row -- A row oriented table widget
//
//    A class specializing in a table of rows.
//    Handles row-specific selection behavior.
//
// Copyright 2002 by Greg Ercolano.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to "erco@seriss.com".
//
//
// TODO:
//    o Row headings (only column headings supported currently)
//

#include <FL/Fl.H>
#include "Fl_Table_Row.H"

// Is row selected?
int Fl_Table_Row::row_selected(int row)
{
    if ( row < 0 || row >= rows() ) return(-1);
    return(rowselect[row]);
}

// Change selection state for row
//
//     flag:
//        0 - clear selection
//        1 - set selection
//        2 - toggle selection
//
//     Returns:
//        0 - selection state did not change
//        1 - selection state changed
//       -1 - row out of range
//
int Fl_Table_Row::select_row(int row, int flag)
{
    if ( row < 0 || row >= rows() ) { return(-1); }

    int oldval = rowselect[row];

    if ( flag == 2 ) { rowselect[row] ^= 1; }
    else             { rowselect[row] = flag; }

    if ( rowselect[row] != oldval )		// select state changed?
    {
	if ( row >= toprow && row <= botrow )	// row visible?
	    { redraw(); }			// redraw
	return(1);
    }
    return(0);
}

// Select all rows to a known state
void Fl_Table_Row::select_all_rows(int flag)
{
    for ( int row=0; row<(int)rowselect.size(); row++ )
	{ rowselect[row] = flag; }
    redraw();
}

// Set number of rows
void Fl_Table_Row::rows(int val)
{
    Fl_Table::rows(val);
    while ( val > (int)rowselect.size() ) { rowselect.push_back(0); }	// enlarge
    while ( val < (int)rowselect.size() ) { rowselect.pop_back(); }	// shrink
}

// #include "eventnames.h"		// debugging
// #include <stdio.h>

// Handle events
int Fl_Table_Row::handle(int event)
{
    // fprintf(stderr, "** EVENT: %s: ", eventnames[event]);	// debugging

    // Let base class handle event
    int ret = Fl_Table::handle(event);
    if ( ret ) return(1);				// base class 'handled' it (eg. column resize)

    static int dragging_select = 0,			// dragging out a selection?
	       last_row = -1,
	       push_row = -1;				// FL_PUSH on this row (used by FL_RELEASE)

    static TableContext push_context = CONTEXT_NONE;	// FL_PUSH context

    int shiftstate = (Fl::event_state() & FL_CTRL) ? FL_CTRL :
                     (Fl::event_state() & FL_SHIFT) ? FL_SHIFT : 0;

    // Which row/column are we over?
    int R, C;  				// row/column being worked on
    ResizeFlag resizeflag;		// which resizing area are we over? (0=none)
    TableContext context = cursor2rowcol(R, C, resizeflag);

    switch ( event )
    {
	case FL_PUSH:

	    // Save for later when button released.
	    //     Do this for all mouse push buttons.
	    //
	    push_context = context;
	    push_row = R;

	    // Handle selection in table.
	    //     Select cell under cursor, and enable drag selection mode.
	    //
	    if ( context == CONTEXT_CELL && Fl::event_button() == 1 )
	    {
		// Ctrl key? Toggle selection state
		switch ( shiftstate )
		{
		    case FL_CTRL:
		        select_row(R, 2);	// toggle
			break;

		    case FL_SHIFT:
		    {
		        select_row(R, 1);
		        if ( last_row > -1 )
			{
			    int srow = R, erow = last_row;
			    if ( srow > erow ) 
				{ srow = last_row; erow = R; }
			    for ( int row = srow; row <= erow; row++ )
				{ select_row(row, 1); }
			}
			break;
		   }

		   default:
		        select_all_rows(0);	// clear all previous selections
			select_row(R, 1);
			break;
		}

		last_row = R;
		dragging_select = 1;
		ret = 1;			// FL_PUSH handled (ensures FL_DRAG will be received)
		// redraw();			// redraw() handled by select_row()
	    }

	    break;

        case FL_DRAG:

	    if ( dragging_select )
	    {
		switch ( shiftstate )
		{
		    case FL_CTRL:
			if ( R != last_row ) 		// toggle only if dragged to new row
			    { select_row(R, 2); } 	// 2=toggle
			break;

		    case FL_SHIFT:
		    default:
			select_row(R, 1);
			if ( last_row > -1 )
			{
			    int srow = R, erow = last_row;
			    if ( srow > erow ) 
				{ srow = last_row; erow = R; }
			    for ( int row = srow; row <= erow; row++ )
				{ select_row(row, 1); }
			}
			break;
		}
		ret = 1;			// drag handled
		last_row = R;
	    }
	    break;

	case FL_RELEASE:
	    if ( Fl::event_button() == 1 )
	    {
		dragging_select = 0;
		ret = 1;			// release handled
	    }
	    break;

	default:
	    break;
    }
    return(ret);
}
